@@ -34,16 +34,35 @@ my $s = Data::Serializer::Sereal->new();
 my %CACHE_COMMON = (
   driver         => 'FastMmap',
   root_dir       => $cache_root->stringify,
-  expires_in     => '5h',
+  expires_in     => '7d',
   cache_size     => '15m',
   key_serializer => $s,
   serializer     => $s,
 );
 
+if ( $ENV{LMDB} ) {
+  eval q<
+    use LMDB_File qw( MDB_NOSYNC MDB_NOMETASYNC );
+    $CACHE_COMMON{'driver'} = 'LMDB';
+
+    $CACHE_COMMON{'flags'} = MDB_NOSYNC | MDB_NOMETASYNC;
+    # STILL SEGVing
+    # $CACHE_COMMON{'single_txn'} = 1;
+    1;
+  > or warn "LMDB Not available $@";
+}
+
 my $get_sha_cache  = CHI->new( namespace => 'get_sha',    %CACHE_COMMON, );
 my $tree_sha_cache = CHI->new( namespace => 'tree_sha',   %CACHE_COMMON, );
 my $meta_cache     = CHI->new( namespace => 'meta_cache', %CACHE_COMMON, );
 
+sub END {
+  undef $get_sha_cache;
+  undef $tree_sha_cache;
+  undef $meta_cache;
+
+  print "Cleanup done\n";
+}
 use Try::Tiny qw( try catch );
 
 sub rev_sha {
@@ -57,16 +76,18 @@ sub rev_sha {
 
 sub tree_sha {
   my ( $sha, $path ) = @_;
-  my $result = $tree_sha_cache->get($sha);
-  return $result if $result;
+  return $tree_sha_cache->compute(
+    $sha, undef,
+    sub {
+      #*STDERR->print("Cache Miss for tree_sha $sha + $path\n");
+      my $tree;
 
-  my $tree;
-
-  try {
-    $tree = [ $git->ls_tree( $sha, $path ) ]->[0];
-  };
-  $tree_sha_cache->set( $sha, $tree );
-  return $tree;
+      try {
+        $tree = [ $git->ls_tree( $sha, $path ) ]->[0];
+      };
+      return $tree;
+    }
+  );
 }
 
 sub file_sha {
@@ -81,39 +102,45 @@ sub file_sha {
 }
 
 sub get_sha {
-  my ($sha)  = @_;
-  my $key    = $sha;
-  my $result = $get_sha_cache->get($key);
-  return $result if defined $result;
-  my $out = join qq[\n], $git->cat_file( '-p', $sha );
-  $get_sha_cache->set( $key, $out );
-  return $out;
+  my ($sha) = @_;
+  my $key = $sha;
+  return $get_sha_cache->compute(
+    $sha, undef,
+    sub {
+      #*STDERR->print("Cache Miss for get_sha $sha\n");
+      return join qq[\n], $git->cat_file( '-p', $sha );
+    }
+  );
 }
 
 sub get_json_prereqs {
   my ($commitish) = @_;
-  my $sha1 = file_sha( $commitish, 'META.json' );
-  if ( defined $sha1 and length $sha1 ) {
-    my $rval = $meta_cache->get($sha1);
-    return $rval if $rval;
-    $rval = CPAN::Meta->load_json_string( get_sha($sha1) );
-    $meta_cache->set( $sha1, $rval );
-    return $rval;
+  if ( $commitish !~ /\d\.\d/ ) {
+    $commitish = rev_sha($commitish);
   }
-  $sha1 = file_sha( $commitish, 'META.yml' );
-  if ( defined $sha1 and length $sha1 ) {
-    my $rval = $meta_cache->get($sha1);
-    return $rval if $rval;
-    $rval = CPAN::Meta->load_yaml_string( get_sha($sha1) );
-    $meta_cache->set( $sha1, $rval );
-    return $rval;
-  }
-  return {};
+  return $meta_cache->compute(
+    $commitish,
+    undef,
+    sub {
+      #*STDERR->print("Cache miss for $commitish metadata\n");
+      my $sha1 = file_sha( $commitish, 'META.json' );
+      if ( defined $sha1 and length $sha1 ) {
+        return CPAN::Meta->load_json_string( get_sha($sha1) );
+      }
+      $sha1 = file_sha( $commitish, 'META.yml' );
+      if ( defined $sha1 and length $sha1 ) {
+        return CPAN::Meta->load_yaml_string( get_sha($sha1) );
+      }
+      return {};
+    }
+  );
 }
 
 my @tags;
 
-for my $line ( reverse $git->RUN( 'log', '--pretty=format:%d', 'releases' ) ) {
+my @lines;
+eval { @lines = reverse $git->RUN( 'log', '--pretty=format:%d', 'releases' ) };
+for my $line (@lines) {
   if ( $line =~ /\(tag:\s*([^ ),]+)/ ) {
     my $tag = $1;
     next if $tag =~ /-source$/;
@@ -239,3 +266,5 @@ $misc->child('Changes.deps.opt')->spew_utf8( _maybe( $changes_opt->serialize ) )
 $misc->child('Changes.deps.dev')->spew_utf8( _maybe( $changes_dev->serialize ) );
 
 path('./Changes')->spew_utf8( _maybe( $master_changes->serialize ) );
+
+1;
